; Copyright (C) 2016-2017 Guido Flohr <guido.flohr@cantanea.com>, 
; all rights reserved.

; This file is distributed under the same terms and conditions as
; Perl itself.

; Prototypes are good and important, contrary to what Conway believes.
[-Subroutines::ProhibitSubroutinePrototypes]

; See https://github.com/rjbs/Dist-Zilla/issues/602
[-TestingAndDebugging::RequireUseStrict]
